;------------------------------------------------------------------------
;
;	RS232 ein und ausgabe routinen
;
;	InitSerial(void)		ringbuffer loeschen
;	PutChar(char ch)		zeichen ausgeben
;	char GetChar(void)		zeichen aus ring puffer lesen
;	char GetNextChar(void)	        naechstes zeichen im ringpuffer lesen
;	word RecBufferCnt(void)         anzahl der zeichen im ringpuffer ermitteln
;	void ClearSerialBuffer(void)	buffer loeschen
;
;------------------------------------------------------------------------

		public	InitSerial,_PutChar,GetChar,RecBufferCnt
		public	GetNextChar,ClearSerialBuffer

BD		equ	0dfh
BAUD2400	equ	0f4h
BAUD9600	equ	0fdh

SMOD		equ	08fh

;------------ die segmente ----------------------------------------------

?PR?rs_int_serv?ring	segment CODE
?BI?ring		SEGMENT BIT
?XD?ring        	SEGMENT XDATA
?DT?ring		SEGMENT DATA


MAX_BUFFER	equ	256				;256 zeichen grosser buffer (z.B.)

;----------- globale variablen ----------------------------------------


	RSEG	?BI?ring
TX_empty:	DBIT	1				;transmitter empty flag

	RSEG    ?DT?ring
RecInCnt:    	ds	2				;counter fuer ringbuffer
RecOutPtr:	ds	2              			;ptr fuer zeichen ausgabe aus ringbuffer
RecInPtr:	ds	2				;ptr fuer zeichen eingabe in den ring buffer
TrnInCnt:	ds	2				;counter fue rzeichen im ausgabebuffer
TrnOutPtr:	ds	2				;ptr fuer zeichen ausgabe 
TrnInPtr:	ds	2				;ptr fuuer zeichen eingabe
	RSEG	?XD?ring
BufferStart:	ds      MAX_BUFFER			;der Receive buffer selber
TrnBuffer:	ds	MAX_BUFFER			;der Transmit buffer

BufferEnd	equ	BufferStart + MAX_BUFFER - 1
TrnBufferEnd	equ	TrnBuffer + MAX_BUFFER - 1


;------------	der interrupt vector ----------------------------------------

		CSEG	at  23h    			;vector besetzen
		ljmp	RsIntService

;------------- ab hier code beginn ----------------------------------------

RSEG	?PR?rs_int_serv?ring


;------------------------------------------------------------------------
;
;	InitSerial(void)
;	ring buffer initialisieren
;
;------------------------------------------------------------------------

InitSerial:	clr	ES				;serieller interrupt aus
		mov	RecInPtr,   #high BufferStart  	;in ptr setzen
		mov	RecInPtr+1, #low  BufferStart
		mov	RecOutPtr,  #high BufferStart  	;out ptr setzen
		mov	RecOutPtr+1,#low  BufferStart
		mov	RecInCnt,#0
		mov	RecInCnt+1,#0			;counter loeschen
		mov	TrnInPtr,   #high TrnBuffer	;in Ptr setzen
		mov	TrnInPtr+1, #low  TrnBuffer
		mov	TrnOutPtr,  #high TrnBuffer	;low Ptr setzen
		mov	TrnOutPtr+1,#low  TrnBuffer
		mov	TrnInCnt,#0			;counter loeschen
		mov	TrnInCnt+1,#0

		setb	TX_empty			;senderegister frei
		clr	BD				;Timer 1 = baudrategenerator
		mov	SCON,#70h			;8 bit uart einstellen
		clr	SMOD
		mov	a,TMOD				;Timer 1 = 8 bit reload mode
		anl	a,#0fh
		orl	a,#20h
		mov	TMOD,a
;		mov	TH1,#BAUD2400			;2400 Baud einstellen
		mov	TH1,#BAUD9600
		setb	TR1
		setb	ES				;serieller interrupt wiederein
		ret

;------------------------------------------------------------------------
;	void	ClearSerialBuffer(void)
;------------------------------------------------------------------------
ClearSerialBuffer:
		clr	ES				;serieller interrupt aus
		mov	RecInPtr,   #high BufferStart  	;in ptr setzen
		mov	RecInPtr+1, #low  BufferStart
		mov	RecOutPtr,  #high BufferStart  	;out ptr setzen
		mov	RecOutPtr+1,#low  BufferStart
		mov	RecInCnt,#0
		mov	RecInCnt+1,#0			;counter loeschen
		mov	TrnInPtr,   #high TrnBuffer	;in Ptr setzen
		mov	TrnInPtr+1, #low  TrnBuffer
		mov	TrnOutPtr,  #high TrnBuffer	;low Ptr setzen
		mov	TrnOutPtr+1,#low  TrnBuffer
		mov	TrnInCnt,#0			;counter loeschen
		mov	TrnInCnt+1,#0
		setb	ES				;serieller interrupt wiederein
		ret
;----------------------------------------------------------------------------
;
;	PutChar(char ch)
;
;	zeichen ausgabe auf serielle schnittstelle
;	ohne ringbuffer
;
;----------------------------------------------------------------------------

_PutChar:	jb	TX_empty,SingleChar		;puffer leer und uart frei

		clr	ES				;ruhe jetzt
		setb	c
		mov	a,TrnInCnt+1
		subb	a,#low (MAX_BUFFER - 1)
		mov	a,TrnInCnt
		subb    a,#high (MAX_BUFFER - 1)
		jc	PutChar1			;buffer ist nicht voll
		setb	ES
		sjmp	_PutChar			;wenn voll warten bis platz da ist

PutChar1:	mov	a,TrnInCnt+1			;wenn nicht voll dann counter incrementieren
		add	a,#1
		mov	TrnInCnt+1,a
		mov	a,TrnInCnt
		addc	a,#0
		mov	TrnInCnt,a

		mov	a,r7				;zeichen holen
		mov	dph,TrnInPtr			;ptr laden und
		mov	dpl,TrnInPtr+1
		movx	@dptr,a				;zeichen speichern

		inc	dptr				;ptr + 1

;	test ob ptr am bufferende angekommen ist

		setb	c
		mov	a,dpl				;ist ptr am buffer ende angekommen ?
		subb	a,#low TrnBufferEnd
		mov	a,dph
		subb	a,#high TrnBufferEnd
		jc	PutChar2			;ist nicht am ende

;	wenn ptr am ende angekommen ist ptr neu setzen

		mov	TrnInPtr,  #high TrnBuffer
		mov	TrnInPtr+1,#low  TrnBuffer
		sjmp	PutChar4

;	ptr wieder abspeichern

PutChar2:	mov	TrnInPtr,dph
		mov	TrnInPtr+1,dpl

PutChar4:	setb	ES				;interrupt freigeben
		ret	

SingleChar:	clr	ES				;ruhe jetzt
		mov	SBUF,r7				;zeichen zum uart
		clr	TX_empty			;flag loeschen
		setb	ES				;interrupt freigeben
		ret

;----------------------------------------------------------------------------
;
;	anzahl der zeichen im RX buffer ermitteln
;
;	word	RecBufferCnt(void)
;
;----------------------------------------------------------------------------

RecBufferCnt: 	mov	r6,RecInCnt
		mov	r7,RecInCnt+1
		ret

;----------------------------------------------------------------------------
;
;	void	GetNextChar(void)
;
;	naechstes zeichen im ringbuffer lesen ohne es im buffer zu loeschen
;
;----------------------------------------------------------------------------

GetNextChar:   	mov	dph,RecOutPtr
		mov	dpl,RecOutPtr+1
		movx	a,@dptr
		mov	r7,a
		ret


;----------------------------------------------------------------------------
;
;	char GetChar
;
;	zeichen aus ring buffer auslesen
;
;----------------------------------------------------------------------------

GetChar:      	mov	a,RecInCnt			;zeichen im buffer ?
		orl	a,RecInCnt+1
		jz	GetChar				;wenn nein warten

		clr	ES				;es ist was da -> int sperren
		mov	dph,RecOutPtr			;ptr laden
		mov	dpl,RecOutPtr+1

		movx	a,@dptr				;zeichen aus ring buffer holen
		mov	r7,a				;und in R7 speichern

		inc	dptr				;ptr auf naechste position

;	test ob am buffer ende angekommen

		setb	c
		mov	a,dpl
		subb	a,#low BufferEnd		;ptr > BufferEnd
		mov	a,dph
		subb	a,#high BufferEnd
		jc	GetChar_1			;nein

;	wenn ja ptr neu laden

		mov	RecOutPtr  ,#high BufferStart
		mov	RecOutPtr+1,#low  BufferStart
		sjmp	GetChar_2

;	sonst ptr speichern

GetChar_1:	mov	RecOutPtr  ,dph			;ptr abpspeichern
		mov	RecOutPtr+1,dpl
GetChar_2:
		mov	a,RecInCnt+1			;counter - 1
		clr	c
		subb	a,#1
		mov	RecInCnt+1,a
		mov	a,RecInCnt
		subb	a,#0
		mov	RecInCnt,a

		setb	ES				;interrupt wieder freigeben
		ret

;-------------------------------------------------------------------------
;
;	service routine fuer senden und empfangen
;
;-------------------------------------------------------------------------


RsIntService:	push    psw
		push	acc
		push	dpl
		push	dph

;	test ob senderegister leer ist wenn ja TX_empty flag setzen

		jnb	TI,int_ser_1		;transmitter interrupt ?
		clr	TI			;YES TI flag loeschen

		mov	a,TrnInCnt		;noch zeichen im trn buffer ?
		orl	a,TrnInCnt+1
		jz	int_ser_trn_1		;nein dann fertig
		
		clr	c
		mov	a,TrnInCnt+1		;counter - 1
		subb	a,#1
		mov	TrnInCnt+1,a
		mov	a,TrnInCnt
		subb	a,#0
		mov	TrnInCnt,a

		mov	dph,TrnOutPtr		;ptr laden
		mov	dpl,TrnOutPtr+1
		movx	a,@dptr			;zeichen aus buffer holen
		mov	SBUF,a			;und zum uart ausgeben

		inc	dptr			;ptr auf naechste position

;	test ob am buffer ende angekommen

		setb	c
		mov	a,dpl
		subb	a,#low TrnBufferEnd	;ptr > BufferEnd
		mov	a,dph
		subb	a,#high TrnBufferEnd
		jc	int_ser_trn_2		;nein

;	wenn ja ptr neu laden

		mov	TrnOutPtr  ,#high TrnBuffer
		mov	TrnOutPtr+1,#low  TrnBuffer
		sjmp	int_ser_1

;	sonst ptr speichern

int_ser_trn_2:	mov	TrnOutPtr  ,dph		;ptr abpspeichern
		mov	TrnOutPtr+1,dpl
		sjmp	int_ser_1

int_ser_trn_1:	setb    TX_empty		;TX_empty flag setzen

;	es ist ein zeichen im uart -> einlesen und im ringbuffer abspeichern

int_ser_1:	jnb	RI,int_serv_ende
		mov	a,SBUF			;zeichen einlesen
		clr	RI			;flag loeschen
		mov	dph,RecInPtr		;ptr laden und
		mov	dpl,RecInPtr+1
		movx	@dptr,a			;zeichen speichern

		inc	dptr			;ptr + 1

;	test ob ptr am bufferende angekommen ist

		setb	c
		mov	a,dpl			;ist ptr am buffer ende angekommen ?
		subb	a,#low BufferEnd
		mov	a,dph
		subb	a,#high BufferEnd
		jc	int_ser_2		;ist nicht am ende

;	wenn ptr am ende angekommen ist ptr neu setzen

		mov	RecInPtr,  #high BufferStart
		mov	RecInPtr+1,#low BufferStart
		sjmp	int_ser_3

;	ptr wieder abspeichern

int_ser_2:	mov	RecInPtr,dph
		mov	RecInPtr+1,dpl


;	buffer voll ?

int_ser_3:	setb	c
		mov	a,RecInCnt+1
		subb	a,#low (MAX_BUFFER - 1)
		mov	a,RecInCnt
		subb    a,#high (MAX_BUFFER - 1)
		jc	int_ser_4		;buffer ist nicht voll

;	buffer ist voll dann RecOutPtr auf aeltestes element setzen

		mov	RecOutPtr  ,RecInPtr
		mov	RecOutPtr+1,RecInPtr+1
		sjmp	int_serv_ende

;	counter incrementieren	wenn buffer noch nicht voll ist

int_ser_4:	mov	a,RecInCnt+1
		add	a,#1
		mov	RecInCnt+1,a
		mov	a,RecInCnt
		addc	a,#0
		mov	RecInCnt,a

int_serv_ende:	pop	dph
		pop	dpl
		pop	acc
		pop	psw
		reti

		end
