;--------------------------------------------------------------------------
;
;	EEprom ansteuerung fuer serielles I2C eeprom 24C08
;
;	void InitEeprom(void)
;		treiber initialisieren
;	byte RdEeprom(word addr)
;		ein byte aus dem eeprom lesen
;	void WrEeprom(word addr,byte val)
;		ein byte im eeprom abspeichern
;
;
;--------------------------------------------------------------------------

		public	InitEeprom,_RdEeprom,_WrEeprom

SDA		equ	P3.5
SCL		equ	P3.4
DeviceType	equ	10100000b
WriteCommand	equ	11111110b
ReadCommand	equ	00000001b


?PR?eeprom?eeprom	segment CODE

RSEG	?PR?eeprom?eeprom


StartCondition	macro
		setb	SDA
		nop
		nop
		setb	SCL
		nop
		nop
		clr	SDA		;SDA zuerst auf 0
		nop
		nop
		clr	SCL		;dann SCL auf 0 legen
		endm

StopCondition	macro
		clr	SDA
		nop
		nop
		setb	SCL		;zuerst SCL auf 1
		nop
		nop
		setb	SDA		;dann SDA auf 1 legen
		endm


InitEeprom:	setb	SDA		; beide leitungen auf 1 legen
		setb	SCL
		ret


_RdEeprom:	setb	P1.7		; keyboard lines auf 1 setzen
		setb	P1.6
		setb	P1.5
		setb	P1.4
		setb	P1.3
		setb	P1.2
		StartCondition		;startcondition ausgeben
		mov	a,r6		;high byte der addresse in R6
		rl	a		;auf position schieben
		anl	a,#00000110b	;bit maskieren
		orl	a,#DeviceType	;device type code setzen
		anl	a,#WriteCommand	;schreib commando setzen
		lcall	SendByte	;zum eeprom ausgeben
		jc	RdError		;Error ?
		mov	a,r7		;low byte der addresse ausgeben
		lcall	SendByte
		jc	RdError		;error ?

		StartCondition

		mov	a,r6		;high byte der addresse in R6
		rl	a		;auf position schieben
		anl	a,#00000110b	;bit maskieren
		orl	a,#DeviceType	;device type code setzen
		orl	a,#ReadCommand	;lese commando setzen
		lcall	SendByte	;zum eeprom ausgeben
		jc	RdError		;Error ?
		lcall	ReadByte	;byte einlesen
		mov	r7,a		;nach R7 senden als return wert

		clr	SDA		;SDA = 0

RdError:	StopCondition		;stop condition ausgeben
		ret


_WrEeprom:	setb	P1.7		; keyboard lines auf 1 setzen
		setb	P1.6
		setb	P1.5
		setb	P1.4
		setb	P1.3
		setb	P1.2
		StartCondition		;Startcondition ausgeben
		mov	a,r6		;high byte der addresse in R6
		rl	a		;auf position schieben
		anl	a,#00000110b	;bit maskieren
		orl	a,#DeviceType	;device type code setzen
		anl	a,#WriteCommand	;schreib commando setzen
		lcall	SendByte	;zum eeprom ausgeben
		jc	WrError		;Error ?
		mov	a,r7		;low byte der addresse ausgeben
		lcall	SendByte
		jc	WrError		;error ?
		mov	a,r5		;daten ausgeben
		lcall	SendByte
WrError:	StopCondition		;stop condition ausgeben
		lcall	Delay		;schreib delay abwarten
		ret


;--------------------------------------------------------------------------
;
;	ein byte vom eeprom einlesen in Acc
;
;--------------------------------------------------------------------------
ReadByte:	mov	r1,#8		;bit counter laden
		setb	SDA		;sda auf eingang stellen
ReadByte_1:	setb	SCL		;clock auf 1 setzen
		nop
		nop
		mov	c,SDA		;sda pin einlesen
		rlc	a		;bit in Acc schieben MSB first
		clr	SCL		;clock auf 0 setzen
		djnz	r1,ReadByte_1	;alle eingelesen
		nop
		setb	SCL		;9 clock impuls ausgeben
		nop
		nop
		clr	SCL		;und wieder weg
		ret

;--------------------------------------------------------------------------
;
;	byte in ACC zum eeprom senden
;	Ack status vom eeprom in CY
;
;--------------------------------------------------------------------------

SendByte:	mov	r1,#8		;bit counter laden
SendByte_1:	rlc	a		;MSB first ins CY
		mov	SDA,c		;dann zum SDA Pin
		nop
		nop
		setb	SCL		;clock = 1
		nop
		nop
		clr	SCL		;clock = 0
		djnz	r1,SendByte_1	;alle ausgesendet ?

		nop
		nop
		setb	SDA		;sda auf eingang schalten
		setb	SCL		;9 clock puls fuer ACK vom EEprom
		mov	c,SDA		;ack status in CY schieben
		nop
		nop
		clr	SCL		;clock = low
		ret

;--------------------------------------------------------------------------
;
;	5 mS delay routine
;
;--------------------------------------------------------------------------
Delay:		mov	R0,#10
Delay_2:	mov	R1,#0
Delay_1:	djnz	R1,Delay_1
		djnz	R0,Delay_2
		ret


		end

