#include <reg52.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include "lcd.h"
#include "eeprom.h"

#ifndef uint
#define uint unsigned int
#endif
#ifndef uchar
#define uchar unsigned char
#endif


#define CURSORDOWN 'X' - 0x40
#define CURSORUP   'E' - 0x40
#define CURSORLEFT 'S' - 0x40
#define CURSORRIGHT 'D' - 0x40
#define ESC 0x1b
#define	MENU 'M'



extern	void	InitBaudot(void);
extern	void	PutChar(uchar);
extern	uchar	GetChar(void);
extern	uint	RecBufferCnt(void);

extern	void	InitSerial(void);
extern	uint	BaudotCnt(void);
extern	uchar	GetBaudotChar(void);

extern	char	BaudotToAscii(char);

extern	void	InitCollect(uint);
extern	void	StoreSearchPattern(uint,uchar*);
extern	void	CollectData(uchar);
extern	uchar*	GetBufferPtr(uint);
extern	uchar	GetUpdateFlag(uint);

#define MAXTERMLINES 200				/* anzahl der zeilen im Scroll Buffer */
#define PATTERNSIZE 21
#define PATTERN1 0
#define PATTERN2 PATTERN1+PATTERNSIZE

code	Str40 Menu1[5] = {"Suchmuster 1 eingeben",
						  "Suchmuster 2 eingeben",
						  "Speicher 1 ausgeben",
						  "Speicher 2 ausgeben",
						  "Laufende Daten ausgeben"};



uchar	ucBigBuffer[0x7800];			// Buffer fuer malloc
uchar	ucLastKey;						// letzte gedrueckten tastencode
uchar	szPattern1[PATTERNSIZE];		// suchmuster 1
uchar	szPattern2[PATTERNSIZE];		// suchmuster 2
uchar	ucComMode;						// mode der seriellen schnittstelle
uchar	szComStr[41];					// eingabe string fuer die serielle schnittstelle
uint	uiComPos;						// position im string

uchar	GetKey(void);
uchar	DebounceKey(void);
void	ShowCurrentData(void);			// momentan einlaufende daten ausgeben auf display
void	ShowBufferData(uint);			// daten in den beiden Buffer ausgeben auf Display
void	ShowTextScreen(uchar*);			// ein displayscreen ausgeben
void	CheckComMode(void);				// uebertragungs mode der seriellen schnittstelle
void	ReadEepromString(uint,uchar*,uint);
void	WriteEepromString(uint,uchar*,uint);

void	Strout(char*);

void main(void)
{
	uint	uiSelect;

	init_mempool(ucBigBuffer,sizeof(ucBigBuffer));
	InitSerial();
	InitBaudot();
	InitEeprom();
	LCDTextDisplayInit();
	LCDTextClearScreen();
	LCDVScrollInitTerminal(MAXTERMLINES);
	InitCollect(0x1800);

	ReadEepromString(PATTERN1,szPattern1,sizeof(szPattern1)-1);
	ReadEepromString(PATTERN2,szPattern2,sizeof(szPattern2)-1);
	StoreSearchPattern(1,szPattern1);
	StoreSearchPattern(2,szPattern2);

	ucLastKey = 0;
	ucComMode = 1;
	uiComPos = 0;
	szComStr[0] = 0;

	EA = 1;

	for(;;)
	{
		ShowCurrentData();
		LCDDisplayInit();
		LCDClearGraphic();
		uiSelect = 0;
		if(LCDListBox("Auswahl",Menu1,5,&uiSelect))
		{
			switch(uiSelect){
			case 0:
				ReadEepromString(PATTERN1,szPattern1,sizeof(szPattern1)-1);
				if(LCDEnterString("Suchmuster 1",szPattern1,sizeof(szPattern1)-1))
				{
					StoreSearchPattern(1,szPattern1);
					WriteEepromString(PATTERN1,szPattern1,sizeof(szPattern1)-1);
				}
				LCDTextDisplayInit();
				break;
			case 1:
				ReadEepromString(PATTERN2,szPattern2,sizeof(szPattern2)-1);
				if(LCDEnterString("Suchmuster 2",szPattern2,sizeof(szPattern2)-1))
				{
					StoreSearchPattern(2,szPattern2);
					WriteEepromString(PATTERN2,szPattern2,sizeof(szPattern2)-1);
				}
				LCDTextDisplayInit();
				break;
			case 2:
				LCDTextDisplayInit();
				ShowBufferData(1);
				break;
			case 3:
				LCDTextDisplayInit();
				ShowBufferData(2);
				break;
			case 4:			// laufende daten ausgabe nix tun nur nach oben springen
				break;
			}
		}
	}
}
/************************************************************************************************
*
*	void	CheckComMode(void)
*	serielle schnittstelle abfragen welches ausgabeformat gewuenscht ist
*
*************************************************************************************************/
void	CheckComMode(void)
{
	uchar	*pPtr;
	uchar	ch;
	uint	n;

	// buffer ausgeben wenn daten upgedatet werden
	if(ucComMode != 1)
	{
		if(GetUpdateFlag((uint)ucComMode-1))
		{
			pPtr = GetBufferPtr((uint)ucComMode-1);
			while(*pPtr)
			{
				if(*pPtr == LF) PutChar(CR);
				PutChar(*pPtr++);
			}
		}
	}

	if(RecBufferCnt())		// zeichen eingetroffen ?
	{
		switch(ch = GetChar()) {
		case LF: 
		//	PutChar(LF);
			return;
		case BS:
			if(uiComPos > 0)
			{
				uiComPos--;
				szComStr[uiComPos] = 0;
				PutChar(BS);
				PutChar(' ');
				PutChar(BS);
			}
			return;
		case CR:
			PutChar(CR);
			PutChar(LF);
			uiComPos = 0;
			break;
		default:
			if(uiComPos < (sizeof(szComStr)-1))
			{
				if(islower(ch)) 
					ch = toupper(ch);
				PutChar(ch);
				szComStr[uiComPos++]=ch;
				szComStr[uiComPos] = 0;
			}
			return;
		}
	} else return;


	pPtr = szComStr;

	while(*pPtr == ' ') pPtr++;

	// buffer 1 ausgeben ?
	if(!strncmp(pPtr,"GB1",3))
	{
		ucComMode = 2;
		pPtr = GetBufferPtr(1);
		Strout("BUFFER1:\r\n");
		while(*pPtr)
		{
			if(*pPtr == LF) PutChar(CR);
			PutChar(*pPtr++);
		}
		Strout("\r\n");
	} else
	// Buffer 2 ausgeben ?		
	if(!strncmp(pPtr,"GB2",3))
	{
		ucComMode = 3;
		pPtr = GetBufferPtr(2);
		Strout("BUFFER2:\r\n");
		while(*pPtr)
		{
			if(*pPtr == LF) PutChar(CR);
			PutChar(*pPtr++);
		}
		Strout("\r\n");
	} else
	// continues Mode ?
	if(!strncmp(pPtr,"CM",2))
	{
		ucComMode = 1;
	} else
	// such pattern1 ausgeben ?
	if(!strncmp(pPtr,"GP1",3))
	{
		Strout(szPattern1);
		Strout("\r\n");
	} else	
	// such pattern2 ausgeben ?
	if(!strncmp(pPtr,"GP2",3))
	{
		Strout(szPattern2);
		Strout("\r\n");
	} else
	// Such Pattern1 eingeben ?
	if(!strncmp(pPtr,"SP1",3))
	{
		pPtr+=3;
		while(*pPtr == ' ') pPtr++;
		n = 0;
		while(*pPtr && n < (sizeof(szPattern1)-1))
		{
			szPattern1[n++] = *pPtr++;
		}
		szPattern1[n] = 0;
		StoreSearchPattern(1,szPattern1);
		WriteEepromString(PATTERN1,szPattern1,sizeof(szPattern1)-1);
	} else
	// Such Pattern2 eingeben ?
	if(!strncmp(pPtr,"SP2",3))
	{
		pPtr+=3;
		while(*pPtr == ' ') pPtr++;
		n = 0;
		while(*pPtr && n < (sizeof(szPattern2)-1))
		{
			szPattern2[n++] = *pPtr++;
		}
		szPattern2[n] = 0;
		StoreSearchPattern(2,szPattern2);
		WriteEepromString(PATTERN2,szPattern2,sizeof(szPattern2)-1);
	}

}
/************************************************************************************************
*
*	void	ShowBufferData(uint uiNo)
*	Daten aus den Buffer ausgeben
*
*************************************************************************************************/
void	ShowBufferData(uint uiNo)
{
	uchar	*pPtr,*p1,*p2,ch;
	uchar	ucFlag;

	LCDTextClearScreen();
	pPtr = GetBufferPtr(uiNo);
	ucFlag = 1;
	for(p1 = pPtr;;)
	{
		if(GetUpdateFlag(uiNo))
		{
			p1 = pPtr;
			ucFlag = 1;
		}
		if(ucFlag)
		{
			ucFlag = 0;
			ShowTextScreen(p1);
		}
		switch(GetKey()) {			// tastenfeld bedienen
		case CURSORUP:
			p2 = p1;
			if (p2 > pPtr)
			{
				if(*(p2-1) == LF) 
				{
					p2-=2;
					while(p2 > pPtr && *p2 != LF) p2--;
					if(*p2 == LF) p2++;
					p1 = p2;
					ucFlag = 1;
				}
			}
			break;
		case CURSORDOWN:			// naechstes LF suchen im buffer
			p2 = p1;
			while(*p2 != LF && *p2 != 0) p2++;
			if(*p2 == LF)
			{
				if(*(p2+1) != 0)
				{
					p1 = p2+1;
					ucFlag = 1;
				}
			}
			break;
		case ESC:
			LCDTextClearScreen();
			return;
		}
		// daten im hintergrund weiter sammeln
		if(BaudotCnt())				// zeichen einlesen
		{
			ch = GetBaudotChar();
			ch = BaudotToAscii(ch);	// nach ascii wandeln

			if(isprint(ch) || ch == CR || ch == LF)
			{
				if(ucComMode == 1)
					PutChar(ch);		// auf schnittstelle ausgeben 
				CollectData(ch);		// daten sammeln
			}
		}
		CheckComMode();
	}
}
/************************************************************************************************
*
*	void	ShowTextScreen(uchar * pPtr)
*	aus dem Buffer einen Screen ausgeben
*
*************************************************************************************************/
void	ShowTextScreen(uchar * pPtr)
{
	uchar	x,y;
	// screen ausgeben bis display ende oder Buffer ende
	for(y=0;y < MAXTEXTY;y++)
	{
		// zeile bis ende oder LF ausgeben
		for(x=0;x < MAXTEXTX;x++)
		{
			if(*pPtr == LF || *pPtr == 0) break;
			LCDTextCharOut(x,y,*pPtr++,TEXTNORMAL);
		}
		// eventuellen rest der zeile loeschen
		for(;x < MAXTEXTX;x++)
			LCDTextCharOut(x,y,' ',TEXTNORMAL);
		if(*pPtr == LF) pPtr++;
		if(*pPtr == 0) break;
	}
	// eventuellen rest des displays loeschen
	for(y++;y < MAXTEXTY;y++)
	{
		for(x=0;x < MAXTEXTX;x++)
			LCDTextCharOut(x,y,' ',TEXTNORMAL);
	}
}
/************************************************************************************************
*
*	void	ShowCurrentData(void)
*	momentan einlaufende Daten ausgeben
*
*************************************************************************************************/
void	ShowCurrentData(void)
{
	uint	uiLineNo;
	uchar	ch;

	uiLineNo = MAXTERMLINES - MAXTEXTY;

	for(;;)
	{
		switch(GetKey()) {			// tastenfeld bedienen
		case CURSORUP:
			if(uiLineNo > 0) uiLineNo--;
			LCDUpDateDisplay(uiLineNo);
			break;
		case CURSORDOWN:
			if(uiLineNo < (MAXTERMLINES-MAXTEXTY)) uiLineNo++;
			LCDUpDateDisplay(uiLineNo);
			break;
		case MENU:
			return;
		}
		if(BaudotCnt())				// zeichen einlesen
		{
			ch = GetBaudotChar();
			ch = BaudotToAscii(ch);	// nach ascii wandeln

			if(isprint(ch) || ch == CR || ch == LF)
			{
				LCDVScrollTerminal(ch);	// auf display ausgeben
				if(ucComMode == 1)
					PutChar(ch);		// auf schnittstelle ausgeben 
				CollectData(ch);		// daten sammeln
			}
		}
		CheckComMode();
	}	
}
/************************************************************************************************
*
*	uchar	GetKey(void)
*	Tasten feld abfragen
*	return: tastenkode oder 0 fuer keine Taste gedrueckt
*
*************************************************************************************************/
uchar	GetKey(void)
{
	uchar	ucKey;

/*	if(RecBufferCnt())
	{
		return(GetChar());
	} else 
		return(0);
*/
	ucKey = DebounceKey();
	if (ucKey != ucLastKey)		// taste gedrueckt oder losgelassen ?
	{
		ucLastKey = ucKey;
		if(ucKey & 0x01) return(CURSORUP);
		if(ucKey & 0x02) return(CURSORDOWN);
		if(ucKey & 0x04) return(CURSORLEFT);
		if(ucKey & 0x08) return(CURSORRIGHT);
		if(ucKey & 0x10) return(CR);
		if(ucKey & 0x20) return(ESC);
		if(ucKey & 0x40) return(MENU);
	}
	return(0);
}
/************************************************************************************************
*
*	uchar	DebounceKey(void)
*	Tasten entprellt einlesen
*				   
*************************************************************************************************/
uchar DebounceKey(void)
{
	uchar	ucKey,n;

	do 
	{
		ucKey = ~P1;
		for(n=0;n<140;n++);
	} while(ucKey != ~P1);
	return(ucKey);
}
/************************************************************************************************
*
*	void	WriteEepromString(uint uiAddr,uchar szStr,uint uiMaxSize);
*	einen string im eeprom speichern
*				   
*************************************************************************************************/
void	WriteEepromString(uint uiAddr,uchar * szStr,uint uiMaxSize)
{
	while(*szStr && uiMaxSize > 0)
	{
		uiMaxSize--;
		WrEeprom(uiAddr++,*szStr++);
	}
	WrEeprom(uiAddr,0);
}
/************************************************************************************************
*
*	void	ReadEepromString(uint uiAddr,uchar szStr,uint MaxSize);
*	einen string im eeprom speichern
*				   
*************************************************************************************************/
void	ReadEepromString(uint uiAddr,uchar * szStr,uint uiMaxSize)
{
	uchar	ch;

	while((ch = RdEeprom(uiAddr++)) != 0 && uiMaxSize > 0)
	{
		if(ch >= 127 || ch < ' ') ch = 0;
		*szStr++ = ch;
		uiMaxSize--;
	}
	*szStr = 0;
}

/************************************************************************************************
*
*	void	Strout(char * Str)
*	String ausgabe routine
*				   
*************************************************************************************************/
void	Strout(char * Str)
{
	while(*Str)
		PutChar(*Str++);
}



